#include "GPUFontLayer.h"
#include "Atlas.h"
#include "AtlasLayerCombo.h"
#include "Utils.h"
#include "font.hpp"
#include "fwd.hpp"
#include "ofGraphics.h"
#include "ofUtils.h"

namespace ofxVariableLab {

void GPUFontLayer::setup(const LayerSettings & settings){
    this->settings = settings;
    if(id == ""){
        id = "layer-" + ofToString(Layer::n_layers);
        n_layers++;
    }
    innerNode.setParent(outerNode);
}

void GPUFontLayer::update(){
    if(propsBuffer.size() > 0){
        color = glm::vec4(propsBuffer[0].color[0],
                          propsBuffer[0].color[1],
                          propsBuffer[0].color[2],
                          propsBuffer[0].color[3]);
    }
}

void GPUFontLayer::draw(glm::vec3 position){
}
void GPUFontLayer::drawCharacter(const char character,
                                 glm::vec3 position,
                                 glm::vec4 color,
                                 float rotation,
                                 float scale,
                                 ofxVariableLab::FontVariation fontVariation){
}
const LayerType & GPUFontLayer::getType() const {
    return type;
}
void GPUFontLayer::setVFlip(const VFlipState vFlip){
    this->vFlip = vFlip;
}

bool GPUFontLayer::isDirtyDirty() const {
    return isDirty;
}

void GPUFontLayer::setDirtyDirty(bool dirtyDirty){
    isDirty = dirtyDirty;
}

bool GPUFontLayer::wantsNewMom(){
    return notHappyWithMom;
}
void GPUFontLayer::isWithNewMom(){
    notHappyWithMom = false;
}
const ComboIdentifier & GPUFontLayer::getMomsComboIdentifier() const {
    return momsComboIdentifier;
}
void GPUFontLayer::setMomsComboIdentifier(const ComboIdentifier & identifier){
    momsComboIdentifier = identifier;
}

void GPUFontLayer::setProps(const Props & props){
    if(propsBuffer.size() == 0 || props.text != propsBuffer[0].text
       || propsBuffer[0].fontVariations != props.fontVariations){
        setDirtyDirty(true);
        variationText.clear();
        variationText.resize(props.text.size());
        int i = 0;
        for(const char c : props.text){
            ofxGPUFont::GlyphIdentity glyphIdentity;
            glyphIdentity.charcode = ofxGPUFont::decodeCharcode(&c);
            for(const auto & fv : props.fontVariations){
                glyphIdentity.coords.push_back(
                    std::move(ofxGPUFont::float2f16dot16(round(fv.value)))     // convert to FT
                    // NOTE: we are also rounding here, since in practice a variation
                    // of 100.0 will not look much different than 100.124
                    );
            }
            variationText[i] = std::move(glyphIdentity);
            i++;
        }
    }
    if(props.fontPath != propsBuffer[0].fontPath){
        notHappyWithMom = true;
    }

    while(propsBuffer.size() > max(0, int(settings.maxBufferSize - 1))){
        propsBuffer.pop_back();
    }
    propsBuffer.push_front(props);
}
const Layer::Props & GPUFontLayer::getProps() const {
    return propsBuffer[0]; // gets newest
}
const glm::vec4 & GPUFontLayer::getColor() const {
    return color;
}
ofNode & GPUFontLayer::getOuterNode(){
    return outerNode;
}
ofNode & GPUFontLayer::getInnerNode(){
    return innerNode;
}
void GPUFontLayer::clearPropsBuffer(){
    propsBuffer.clear();
}
void GPUFontLayer::setId(const LayerID & id){
    this->id = id;
}
const LayerID & GPUFontLayer::getId(){
    return id;
}
const vector <ofxGPUFont::GlyphIdentity> & GPUFontLayer::getVariationText(){
    return variationText;
}
}
